const Ci = Components.interfaces;
const Cc = Components.classes;
const Cr = Components.results;

const PERMS_FILE      = 0644;
const PERMS_DIRECTORY = 0755;

const MODE_RDONLY   = 0x01;
const MODE_WRONLY   = 0x02;
const MODE_CREATE   = 0x08;
const MODE_APPEND   = 0x10;
const MODE_TRUNCATE = 0x20;

var ENGINES = [];

function onLoad() {
  var pref = Cc["@mozilla.org/preferences-service;1"].
             getService(Ci.nsIPrefBranch);
  var dir = pref.getComplexValue("sherlock.folder", Ci.nsILocalFile);
  if (dir)
    loadSrcFiles(dir);
}

function getFolder() {
  var fp = Cc["@mozilla.org/filepicker;1"].
           createInstance(Ci.nsIFilePicker);
  fp.init(window, "Choose searchplugin directory", Ci.nsIFilePicker.modeGetFolder);

  if (fp.show() == Ci.nsIFilePicker.returnOK)
    return fp.file.clone().QueryInterface(Ci.nsILocalFile);

  return null;
}

function getFileFromPath(aPath) {
  var file = Cc["@mozilla.org/file/local;1"].
             createInstance(Ci.nsILocalFile);
  try {
    file.initWithPath(aPath);
  } catch (ex) {
    return null;
  }
  return file;
}

function startFromFresh() {
  var listbox = document.getElementById("fileList");
	var rowCount = listbox.getRowCount();
  // Clear the listbox
	for (var i=0; i<rowCount; i++)
    listbox.removeItemAt(0);

  listbox = document.getElementById("loadedList");
  rowCount = listbox.getRowCount();
  // Clear the listbox
	for (var i=0; i<rowCount; i++)
    listbox.removeItemAt(0);

  ENGINES = [];
  SHERLOCK_FILES = [];
}

function loadSrcFiles(aDir) {
  startFromFresh();

  var listbox = document.getElementById("fileList");
  var dir = aDir || getFolder();
  if (!dir || !dir.exists())
    return;

  var pref = Cc["@mozilla.org/preferences-service;1"].
             getService(Ci.nsIPrefBranch);
  pref.setComplexValue("sherlock.folder", Ci.nsILocalFile, dir);

  var files = dir.directoryEntries;
  var ios = Cc["@mozilla.org/network/io-service;1"].
            getService(Ci.nsIIOService);

  while (files.hasMoreElements()) {
    var file = files.getNext().QueryInterface(Ci.nsILocalFile);
    if (!file.isFile())
      continue;

    var fileURL = ios.newFileURI(file).QueryInterface(Ci.nsIURL);
    var fileExtension = fileURL.fileExtension.toLowerCase();
    var fileName = fileURL.fileName;
    if (fileExtension == "src") {
      SHERLOCK_FILES.push(file);
      listbox.appendItem(fileName, SHERLOCK_FILES.length-1);
    }
  }
}

function loadEngines() {
  var listbox = document.getElementById("fileList");
  var listbox2 = document.getElementById("loadedList");
  var listbox3 = document.getElementById("errorList");

  var length=listbox.getRowCount();
  for (var i=length-1; i>=0; i--) {
    var item = listbox.getItemAtIndex(i);
    try {
      ENGINES[i] = new Engine(SHERLOCK_FILES[item.value]);
    } catch (ex) {
      LOG(ex);
      continue;
    }
    listbox.removeItemAt(i);
    var ers = verifyEngine(ENGINES[i]);
    if (ers)
      listbox3.appendItem(item.label, item.value);
    else
      listbox2.appendItem(item.label, item.value);
  }
}

function showFile(aIndex) {
  var file = ENGINES[aIndex]._file;
  if (!file)
    return;
  var text = readFile(file);
  var textbox = document.getElementById("bottomText");
  textbox.value = text;
}

function showEngine(aIndex) {
  var engine = ENGINES[aIndex];
  var textbox = document.getElementById("topText");
  var str = "";
  str += "Name: '" + engine._name + "'\n";
  str += "Description: '" + engine._description + "'\n";
  str += "QueryCharset: '" + engine._queryCharset + "'\n";
  str += "Urls:\n";
  var urls = engine._urls;
  for (var i in urls) {
    var url = urls[i];
    str += "\tType: '" + url.type + "'\n";
    str += "\tMethod: '" + url.method + "'\n";
    str += "\tTemplate: '" + url.template + "'\n";
    str += "\tParams:\n";
    for (var j in url.params)
      str += "\t\t'" + url.params[j].name + "' -> '" + url.params[j].value + "'\n";
  }
  str += "\n";
  str += "Error summary:\n";
  str += verifyEngine(engine) || "\tNo errors!";
  str += "\n";
  textbox.value = str;
}

function verifyEngine(engine) {
  if (!engine)
    return "";
  var file = engine._file;

  var result = "";
  function err(str) {
    result += "\t" + str + "\n";
    return true;
  }

  var url = engine._urls[0];

  // Ensure the URL is valid
  makeURI(url.template) || err("Not a valid URI!");

  // Ensure the EngineURL contains searchTerms
  var foundUserDefined = false;
  var params = url.params;
  for (var i=0; i<params.length && !foundUserDefined; i++)
    foundUserDefined = (params[i].value == kUserDefined);

  foundUserDefined || err("No userdefined search term!");

  // Check the encoding type

  // Check that the method makes sense
  var method = url.method;
  method == "GET" || method == "POST" || err("Invalid method!");

  // Simulate a search:
  function makeQueryString(aParams, aData) {
    var str = "";
    for (var i = 0; i < aParams.length; ++i) {
      var param = aParams[i];
      var value = param.value.replace(kValidWords, aData);
      str += (i > 0 ? "&" : "") + param.name + "=" + value;
    }
    return str;
  }

  var searchURL = url.template.replace(kValidWords, "foo");
  var postData = null;
  
  var data = "foo";

  var textToSubURI = Cc["@mozilla.org/intl/texttosuburi;1"].
                     getService(Ci.nsITextToSubURI);
  try {
    data = textToSubURI.ConvertAndEscape(engine._queryCharset, data);
  } catch (ex) {
    err("charset screwed up!");
    //LOG("getSubmission:\nFalling back to default queryCharset!");
    //data = textToSubURI.ConvertAndEscape(DEFAULT_QUERY_CHARSET, data);
  }

  var dataString = makeQueryString(url.params, data);
  if (url.method == "GET") {
    // GET method requests have no post data, and append the encoded
    // text to the url...
    if (searchURL.indexOf("?") == -1)
      searchURL += "?";
    searchURL += dataString;
  } else {
    // POST method requests must wrap the encoded text in a MIME
    // stream and supply that as POSTDATA.
    var stringStream = Cc["@mozilla.org/io/string-input-stream;1"].
                       createInstance(Ci.nsIStringInputStream);
    stringStream.data = dataString;

    postData = Cc["@mozilla.org/network/mime-input-stream;1"].
               createInstance(Ci.nsIMIMEInputStream);
    postData.addHeader("Content-Type", "application/x-www-form-urlencoded");
    postData.addContentLength = true;
    postData.setData(stringStream);
  }
  makeURI(searchURL) || err("Finalized URL is invalid!");

  return result;
}

function readFile(aFile) {
  var fileInStream = Cc["@mozilla.org/network/file-input-stream;1"].
                     createInstance(Ci.nsIFileInputStream);
  fileInStream.init(aFile, MODE_RDONLY, PERMS_FILE, 0);
  var sis = Cc["@mozilla.org/scriptableinputstream;1"].
            createInstance(Ci.nsIScriptableInputStream);
  sis.init(fileInStream);

  var data = sis.read(sis.available());
  sis.close();
  return data;
}

function jsConsole() {
  var wwatch = Cc["@mozilla.org/embedcomp/window-watcher;1"].
               getService(Ci.nsIWindowWatcher);
  wwatch.openWindow(null, "chrome://global/content/console.xul", "_blank",
                    "chrome,dialog=no,all", null);
}
