const Cc = Components.classes;
const Ci = Components.interfaces;
const Cr = Components.results;

const handler = {

  /* nsISupports */
  QueryInterface: function (aIID) {
    if (aIID.equals(Ci.nsICommandLineHandler) ||
        aIID.equals(Ci.nsIFactory) ||
        aIID.equals(Ci.nsISupports))
      return this;

    throw Cr.NS_ERROR_NO_INTERFACE;
  },

  /* nsICommandLineHandler */
  handle: function (aCmdLine) {
    var file = null;
    var arg;
    for (var i=0; ; i++) {
      try {
        arg = aCmdLine.getArgument(i);
        if (!arg)
          break;
        file = aCmdLine.resolveFile(arg);
        if (file.exists())
          break;
        else
          file = null;
      } catch (e) {
        break;
      }
    }

    var ps = Cc["@mozilla.org/embedcomp/prompt-service;1"].
             getService(Ci.nsIPromptService);
    if (file) {
      file.QueryInterface(Ci.nsILocalFile);
      var pref = Cc["@mozilla.org/preferences-service;1"].
                 getService(Ci.nsIPrefBranch);
      pref.setComplexValue("sherlock.folder", Ci.nsILocalFile, file);
    }
  },

  helpInfo: "-folder the folder to use to load engines from.\n",

  /* nsIFactory */
  createInstance: function (aOuter, aIID) {
    if (aOuter != null)
      throw Cr.NS_ERROR_NO_AGGREGATION;
    return this.QueryInterface(aIID);
  },

  lockFactory: function (aLock) {}
};

const contractID = "@mozilla.org/commandlinehandler/general-startup;1?type=myapp";
const CID = Components.ID("{2991c315-b871-42cd-b33f-bfee4fcbf682}");
const category = "c-SherlockParser";

const module = {
  /* nsISupports */
  QueryInterface: function (aIID) {
    if (aIID.equals(Ci.nsIModule)     ||
        aIID.equals(Ci.nsISupports))
      return this;

    throw Cr.NS_ERROR_NO_INTERFACE;
  },
  
  /* nsIModule */
  getClassObject: function (aCompMgr, aCID, aIID) {
    if (aCID.equals(CID))
      return handler.QueryInterface(aIID);

    throw Cr.NS_ERROR_NOT_REGISTERED;
  },

  registerSelf: function (aCompMgr, aFileSpec, aLocation, aType) {
    aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);
    
    aCompMgr.registerFactoryLocation(CID,
                                     "SherlockParser",
                                     contractID,
                                     aFileSpec,
                                     aLocation,
                                     aType);

    var catMan = Cc["@mozilla.org/categorymanager;1"].
                 getService(Ci.nsICategoryManager);
    catMan.addCategoryEntry("command-line-handler", category, contractID,
                            true, true);
  },
  
  unregisterSelf: function (aCompMgr, aLocation, aType) {
     aCompMgr.QueryInterface(Ci.nsIComponentRegistrar);
  
     aCompMgr.unregisterFactoryLocation(CID, aLocation);
  
     var catMan = Cc["@mozilla.org/categorymanager;1"].
                  getService(Ci.nsICategoryManager);

     catMan.deleteCategoryEntry("command-line-handler", category);
  },
  
  canUnload: function (aCompMgr) {
    return true;
  }
};

function NSGetModule(comMgr, fileSpec) {
  return module;
}